/**
 *  DfpAds
 *  @class Pheme.helpers.DfpAds
 *  Description: Helper methods and functions for managing the DFP ads
 */
Ext.define('Pheme.helpers.DfpAds', {
    singleton: true,
    config: {
        //Check to see if the PhoneGap plugin is present.
        dfpPluginInstance: window.DFPPlugin || undefined,

        //Allows for custom networkID to be specified for Ad Requests
        networkId: undefined
    },

    /**
     * For Ad Debugging mode for AdOps
     * @return {[type]} [description]
     */
    getDebugMode: function() {
        return Pheme.helpers.UserSettings.get('adDebugMode', false);
    },

    /**
     * [setDebugMode description]
     * @param {[type]} val [description]
     */
    setDebugMode: function(val) {
        Pheme.helpers.UserSettings.set('adDebugMode', val);
    },

    /**
     * Enables insertion of test=yes params to Ad Requests
     * @return {[type]} [description]
     */
    getTestAds: function() {
        return Pheme.helpers.UserSettings.get('adTestAds', false);
    },

    /**
     * [setTestAds description]
     * @param {[type]} val [description]
     */
    setTestAds: function(val) {
        Pheme.helpers.UserSettings.set('adTestAds', val);
    },

    /**
     * Enables the google adUnitId instead of the default
     * @return {[type]} [description]
     */
    getSampleAds: function() {
        return Pheme.helpers.UserSettings.get('adSampleAds', false);
    },

    /**
     * [setSampleAds description]
     * @param {[type]} val [description]
     */
    setSampleAds: function(val) {
        Pheme.helpers.UserSettings.set('adSampleAds', val);
    },

    /*
     * Creates a DFP Banner Ad.
     * Note: This does not handle the creation of true interstitial ads. This will only create a Banner Ad or a
     * Big Box ad which we are using as faux-interstitials
     */
    createDfpAd: function(options, removePrevAds) {
        var dfpInst = this.getDfpPluginInstance(),	//We need the DFP Plugin Instance. If it doesn't exist, we can't try to create anything.
            debug = this.getDebugMode(),
            networkId = this.getNetworkId(),
            testAds = this.getTestAds(),
            sampleAds = this.getSampleAds(),
            localOptions = undefined;

        //If no options object is passed in, we create a default options object with the required properties to make an Ad Request.
        if(undefined === options) {
            localOptions = {
                'adUnitId': '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.' + (Ext.os.is.iOS ? 'ip' : 'aroid') + '/news/index',
                'adSize': 'BANNER'
            };
            options = localOptions;
        }

        if(undefined === removePrevAds) {
            removePrevAds = true;
        }

        if(dfpInst) {

            //If AdOps wants to specify a different network ID, we add this to the options object which gets sent to the createBannerAd method.
            if(debug && undefined !== networkId) {
                options['networkId'] = networkId;
            }

            if(testAds) {
                if(typeof options.tags == 'undefined')
                    options.tags = {};

                options.tags.test = 'yes';
            }

            //Sets the debug mode in plugin. This value is used internally to the plugin.
            (debug) ? dfpInst.debugMode({ 'debug': true }) : dfpInst.debugMode({ 'debug': false });

            //For removing existing ads before creating the current one.
            if(removePrevAds) {
                //dfpInst.removeAd();
                Pheme.helpers.DfpAds.removeAds();
            }

            // ver=2 required on all ads
            if(typeof options.tags == 'undefined')
                options.tags = {};

            options.tags.ver = '2';

            // set adUnitIt if they want to see sample ads
            if(sampleAds) {
                var adUnitId = '/6253334/dfp_example_ad/';

                if(options['adSize'] == 'BIGBOX') {
                    adUnitId += 'multisize';
                } 
                else {
                    adUnitId += 'banner';
                }

                options['adUnitId'] = adUnitId;
            }

            dfpInst.createBannerAd(options, this.adCreationSuccessfulCallback, this.adCreationFailureCallback);

            return true;
        } else {
            return false;
        }
    },

    /**
     * [createInterstitialAd description]
     * @param  {[type]} options       [description]
     * @param  {[type]} removePrevAds [description]
     * @return {[type]}               [description]
     */
    createInterstitialAd: function(options, removePrevAds) {
        var dfpInst = this.getDfpPluginInstance(),	//We need the DFP Plugin Instance. If it doesn't exist, we can't try to create anything.
            debug = this.getDebugMode(),
            networkId = this.getNetworkId(),
            testAds = this.getTestAds(),
            sampleAds = this.getSampleAds(),
            localOptions = undefined;

        //If no options object is passed in, we create a default options object with the required properties to make an Ad Request.
        if(!options) {
            localOptions = {
                'adUnitId': '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.' + (Ext.os.is.iOS ? 'ip' : 'aroid') + '/news/index',
            };

            options = localOptions;
        }

        if(dfpInst) {

            //If AdOps wants to specify a different network ID, we add this to the options object which gets sent to the createBannerAd method.
            if(debug && networkId) {
                options['networkId'] = networkId;
            }

            if(testAds) {
                if(typeof options.tags == 'undefined')
                    options.tags = {};
                
                options.tags.test = 'yes';
            }

            //Sets the debug mode in plugin. This value is used internally to the plugin.
            (debug) ? dfpInst.debugMode({ 'debug': true }) : dfpInst.debugMode({ 'debug': false });

            //For removing existing ads before creating the current one.
            if(removePrevAds) {
                dfpInst.removeAd();
            }

            // ver=2 required on all ads
            if(typeof options.tags == 'undefined')
                options.tags = {};

            options.tags.ver = '2';

            if(sampleAds)
                options['adUnitId'] = '/6253334/dfp_example_ad/interstitial';
            
            dfpInst.createInterstitialAd(options, this.adCreationSuccessfulCallback, this.adCreationFailureCallback);

            return true;
        } else {
            return null;
        }
    },

    /*
     * Removes all DFP Banner Ad instances running within app.
     */
    removeAds: function() {
        var dfpInst = this.getDfpPluginInstance();

        if(dfpInst) {
            dfpInst.removeAd();
        } else {
            return null;
        }
    },

    /**
     * [toggleDebugMode description]
     * @return {[type]} [description]
     */
    toggleDebugMode: function() {
        var debug = this.getDebugMode();

        (debug) ? this.setDebugMode(false) : this.setDebugMode(true);
    },

    /**
     * [toggleTestAds description]
     * @return {[type]} [description]
     */
    toggleTestAds: function() {
        var testAds = this.getTestAds();

        (testAds) ? this.setTestAds(false) : this.setTestAds(true);
    },

    /**
     * [toggleSampleAds description]
     * @return {[type]} [description]
     */
    toggleSampleAds: function() {
        var sampleAds = this.getSampleAds();

        (sampleAds) ? this.setSampleAds(false) : this.setSampleAds(true);
    },

    /**
     * [renderBannerAd description]
     * @param  {[type]} story           [description]
     * @param  {[type]} backgroundColor [description]
     * @param  {[type]} clearAds        [description]
     * @param  {[type]} useMainAdCodes  [description]
     * @return {[type]}                 [description]
     */
    renderBannerAd: function(story, backgroundColor, clearAds, useMainAdCodes) {
        if(undefined === story || null === story) {
            return null;
        }

        var storyData,
            bgColor = backgroundColor,
            options;

        if(!bgColor) {
            bgColor = "#252525";
        }

        if(story.data.main_ad_codes && useMainAdCodes) {
            storyData = story.data.main_ad_codes;
        } else {
            storyData = story.data.ad_codes;
        }

        if(storyData) {
            options = {
                'adSize': 'BANNER',
                'tags': {
                    'pr': Pheme.Config.fetchCurrentMarket().adCode,
                    'loc': storyData.loc,
                    'imp': storyData.imp,
                    'page': storyData.page,
                    'ck': storyData.ck,
                    'nk': storyData.nk
                },
                'backgroundColor': bgColor
            };

            options['adUnitId'] = (storyData.ad) ? '/3081/' + Pheme.Config.fetchCurrentMarket().adCode + '.' + (Ext.os.is.iOS ? 'ip' : 'aroid') + '/' + storyData.ad : 'news';
            //This is for sub-category keys in ads
            if(storyData.sck && storyData.sck.length > 0) {
                options['tags']['sck'] = storyData.sck;
            }

            if(Pheme.helpers.DfpAds.getTestAds()) {
                options['tags']['test'] = 'yes';
            }

            Pheme.helpers.DfpAds.createDfpAd(options, clearAds);
        }
    },

    /**
     * [adCreationSuccessfulCallback description]
     * @return {[type]} [description]
     */
    adCreationSuccessfulCallback: function() {
        Pheme.helpers.DfpAds.getDfpPluginInstance().requestAd({isTesting:false}, function() {
            console.log('success');
        }, function() {
            console.log('failed');
        });
        Pheme.helpers.DfpAds.logAdCreationResult('request successful');
    },

    /**
     * [adCreationFailureCallback description]
     * @return {[type]} [description]
     */
    adCreationFailureCallback: function() {
        Pheme.helpers.DfpAds.logAdCreationResult('problem creating ad');
    },

    /**
     * [logAdCreationResult description]
     * @param  {[type]} message [description]
     * @return {[type]}         [description]
     */
    logAdCreationResult: function(message) {
        var debug = this.getDebugMode();
        if(debug) {
            console.log(message);
        }
    },

    /**
     * [constructor description]
     * @param  {[type]} config [description]
     * @return {[type]}        [description]
     */
    constructor: function(config) {
        this.initConfig(config);
    },
    
    /**
     * [init description]
     * @return {[type]} [description]
     */
    init: function() {
        this.callParent(arguments);
    }
});