/*
 * Favorite Stories Helper
 * 
 * This class encapsulates the functionality of favorite stories throughout the app.
 */
Ext.define('Pheme.helpers.FavoriteStories', {
	singleton: true,

	/**
     * Show Favorite Story View
     * Handles the retrieval of stories and UI updates
     */
    showFavoriteView: function() {
        var me = this;
        var favoriteView = Ext.getCmp('FavoriteView');

        // send stats
        Pheme.helpers.Omniture.pageEvent("Alert Icon Click", Ext.Viewport.getActiveItem().id);
        
        // get favorite stories (if they are not in the local 'Stories' store then we will grab them from remote)
        this.getFavoriteStories(function(data) {
            // set the update acknowledged date on favorite stories
            for(var i=0;i<data.length;i++) {
                me.acknowledgeFavoriteStoryUpdate(data[i].data.id);
            }

            // update the favorite list with our new data
            me.updateFavoriteList(data, favoriteView);
                
            // update how-to message
            if(data.length > 0)
                favoriteView.removeCls('breaking-news-no-stories');
            else
                favoriteView.addCls('breaking-news-no-stories');

            // update favorite story count
            var followingCmp = Ext.getCmp("followingCount");
            followingCmp.setHtml(data.length);

            // update icon updated count
			me.updateFavoriteUpdatedCount();
        });

        // show the view
        if(favoriteView.isHidden())
            favoriteView.show(null);
    },

	/**
     * Update the Favorite Story List Data
     * @param array Stories
     * @param object Parent Component to add List to
     */
    updateFavoriteList: function(stories, parentComponent) {
        if(typeof stories == 'undefined') return;

        var favoriteList = Ext.getCmp('FavoriteList');

        // always destroy the list
        if(favoriteList) {
            parentComponent.remove(favoriteList, false);
        }

        if(stories.length > 0) {
            favoriteList = parentComponent.add(this.favoriteListFactory(stories));

            favoriteList.getStore().sort('date', 'DESC');
        }
    },

    /**
     * Generate List of Favorite Stories
     * @param array Story objects
     * @return Object
     */
    favoriteListFactory: function(stories) {
        return Ext.create('Ext.List', {
            id: 'FavoriteList',
            cls: 'alert-list',
            data: this.getStoryDataFromObject(stories),
            disableSelection: true,
            scrollToTopOnRefresh: false,
            grouped: false,
            flex: 1,
            itemTpl: [
                    '<tpl if="updated"><div class="update-story-item"></tpl>',
                        '<div class="follow-stories-item">',
                            '<div class="breaking-padding-wrap">',
                                '<h3 class="list-headline-3">{title}</h3>',
                                '<div class="timestamp">', 
                                    '<div class="following-star"></div>',
                                    '<tpl if="updated">',
                                        '<span class="updated">UPDATED </span>',
                                    '</tpl>',
                                    '<span>{date_utc:date("M j, Y g:i A")}</span>', 
                                '</div>', 
                            '</div>',
                        '</div>',
                    '<tpl if="updated"></div></tpl>',
                    '<div class="follow-stories-item-unfollow">',
                        '<div class="unfollow-button">unfollow</div>',
                    '</div>'
                ].join('')
        });
    },

    /**
     * Store/remove a story as one of your favorites.
     * @param int storyId
     * @return int [-1=removed, 0=error, 1=added]
     */
    toggleFavoriteStory: function(storyId) {
    	var result = 0;

        try { 
            if(!storyId) return result;

            var favoriteStore = Ext.getStore("FavoriteStories");
            var favorite = favoriteStore.getById(storyId);

            if(favorite) {
                favoriteStore.remove(favorite);

                //Remove tag for Urban Airship
                try {
                    Pheme.helpers.UaAlerts.setPushInst(PushNotification);
                    Pheme.helpers.UaAlerts.removeTag(storyId.toString());
                } 
                catch(e) {}

                Pheme.helpers.Omniture.storyAction("Unfollow Click", storyId);

                result = -1;
            }
            else {
                var favDate = new Date();

                var newFav = Ext.create('Pheme.model.FavoriteStory', {
                    id: storyId, 
                    date: favDate
                });

                favoriteStore.add(newFav);
                
                // Had to use set so that the record would be stored
                // on occasion it would not store it without another event to trigger it
                newFav.set('update_last_acknowledged', favDate);

                //Set tag for Urban Airship
                try {
                    Pheme.helpers.UaAlerts.setPushInst(PushNotification);
                    Pheme.helpers.UaAlerts.setTags(storyId.toString());
                } 
                catch(e) {}

                Pheme.helpers.Omniture.storyAction("Follow Click", storyId);

                result = 1;
            }

            // update content lists with changes
            Pheme.helpers.Common.refreshContentLists();

            // update favorite update count
            Pheme.helpers.FavoriteStories.updateFavoriteUpdatedCount();
        }
        catch(e) {}

        return result;
    },

    /**
     * Get a list of favorite stories
     * @param function callback( array Story )
     */
    getFavoriteStories: function(callback) {
        var storyList = [];
        var favoriteStore = Ext.getStore("FavoriteStories");
        var storyStore = Ext.getStore("Stories");
        var centralStore = Ext.getStore("CentralStories");
        var getRemoteStoryIdList = [];

        // check and see if we are missing any favorite stories in the 'Stories' store
        favoriteStore.each(function(item, index, length) {
            var story = storyStore.getById(item.internalId);
            
            // look in central store for the story
            if(!story) {
                story = centralStore.getById(item.internalId);
            }

            if(story) {
                // add date favorited to story for favorite list sorting
                story.data.date_favorited = item.data.date;
                storyList.push(story);
            }
            else {
                getRemoteStoryIdList.push(item.internalId);
            }
        });

        // if we are missing some stories get them from the remote and wait on callback
        // TODO: Remove stories from favorites that no longer exist in WP
        if(getRemoteStoryIdList.length > 0) {
            var me = this;
            Pheme.helpers.Common.getRemoteStories(getRemoteStoryIdList, function(data) {
                me.getFavoriteStories(callback, true);
            });   
        }
        else {
            callback(storyList);
        }
    },

    /**
     * Get the total number of favorite stories.
     * @return int
     */
    favoriteCount: function() {
    	var favoriteStore = Ext.getStore("FavoriteStories");
    	return favoriteStore.getCount();
    },

    /**
     * Update UI with number of favorite stories that have been updated
     */
    updateFavoriteUpdatedCount: function() {
        var me = this;

    	this.getFavoriteStories(function(data) {
    		var followedStoryUpdatedCount = 0;

    		// check and see which stories have been updated
            // and which they are not aware of an update too yet
    		for(var i=0;i<data.length;i++) {
    			if(data[i].data.updated && !me.awareOfUpdateToFavoriteStory(data[i].data)) {
    				followedStoryUpdatedCount++;
                }
    		}

	        var followingTitleCmp = Ext.getCmp("followingCountTitleBar");
	        var followingTitleCmpStory = Ext.select(".follow-count-icon.story");

	        // update UI
	        if(followedStoryUpdatedCount == 0) {
	            if(followingTitleCmp) 
	            	followingTitleCmp.hide();

	            if(followingTitleCmpStory)
                    followingTitleCmpStory.hide();
	        }
	        else {
	            followingTitleCmp.setHtml(followedStoryUpdatedCount);
	            followingTitleCmp.show();

	            followingTitleCmpStory.setHtml(followedStoryUpdatedCount);
	            followingTitleCmpStory.show();
	        }
    	});
    },

    /**
     * Check if story is a favorite.
     * @param int storyId
     * @return bool
     */
    isFavoriteStory: function(storyId) {
        if(!storyId) return false;

        var favoriteStore = Ext.getStore("FavoriteStories");
        var favorite = favoriteStore.getById(storyId);

        return (favorite);
    },

    /**
     * Return the story data for a story object
     * @param mixed Story or Array of Stories
     * @return mixed
     */
    getStoryDataFromObject: function(stories) {
        // must be a single story object
        if(typeof stories.data != 'undefined') {
            return stories.data;
        }
        // must be an array of objects
        else {
            var list = [];

            for(var i=0;i<stories.length;i++) {
                list.push(stories[i].data);
            }

            return list;
        }
    },

    /**
     * Check if the user is aware of the update to the favorite story.
     * @param  {[type]} story [description]
     * @return {[type]}         [description]
     */
    awareOfUpdateToFavoriteStory: function(story) {
        try { 
            if(!story) return false;

            // check favorites for this story
            var favoriteStories = Ext.getStore("FavoriteStories");
            var fav = favoriteStories.getById(story.id);

            // not a favorite
            if(!fav) return false;

            // user has not acknowledged any updates
            if(fav.data.update_last_acknowledged == null) return false;

            // date last acknowledged > date last updated
            return (fav.data.update_last_acknowledged > story.date_utc);
        } 
        catch(e) { return false; }
    },

    /**
     * Acknowledge an update to a story. Used to determine which count to
     * show in the alert icon. Only show a count for updated favorites since
     * the last time you tapped the icon.
     * @param  {[type]} storyId [description]
     * @return {[type]}         [description]
     */
    acknowledgeFavoriteStoryUpdate: function(storyId) {
        try {
            if(!storyId) return false;

            var favoriteStories = Ext.getStore("FavoriteStories");
            var fav = favoriteStories.getById(storyId);
            var newDate = new Date();

            if(!fav) return false;
            fav.set('update_last_acknowledged', newDate);

            return true;
        }
        catch(e) { return false; }
    },
});