/**
 *  
 *  Omniture Stats Wrapper Class
 *  @class Pheme.helpers.Omniture
 *  Description: This is to handle the processing and catch errors with any of the ADB calls
 */
Ext.define('Pheme.helpers.Omniture', {
	singleton: true,

	/*
	 *	Track the tap of a link within a story
	 *	@param string name Name of event
	 *	@param object obj Story Id or Story Object
	 *	@param string link url
	 */
	storyLinkTap: function(name, obj, link) {
		this.storyAction(name, obj, { linkURL: link });
	},

	/*
	 *	Track the press of a button (alert, back, home, etc...) that has story related data
	 *	@param string name Name of event
	 *	@param object obj Story Id or Story Object
	 *	@param object optionalData
	 */
	storyAction: function(name, obj, optionalData) {
		if(typeof name == 'undefined' || typeof obj == 'undefined') return;

		// get track data for story
		var trackData = this.getStoryTrackData(obj);
		
		// add config values
		trackData.distributor = Pheme.Config.fetchCurrentMarket().distributor || '';
	    trackData.newspaperBrand = Pheme.Config.fetchCurrentMarket().newspaperBrand || '';

        // add key/value from name
        // this is what omniture wanted.. be careful not to pass a name that is the same as an existing key
        trackData[name] = name;

        // set optional data
        if(typeof optionalData != 'undefined') {
        	if(typeof optionalData.linkURL != 'undefined')
        		trackData.linkURL = optionalData.linkURL;

        	// note pageName could be set via story data
        	if(typeof optionalData.pageName != 'undefined')
        		trackData.pageName = optionalData.pageName;

        	if(typeof optionalData.searchTerm != 'undefined')
        		trackData.searchTerm = optionalData.searchTerm;
        }

        this.trackAction(name, trackData);
	},

	/*
	 *	Build and return story data needed for tracking
	 *	@param object obj Story Id or Story Object
	 *	@return object trackData
	 */
	getStoryTrackData: function(obj) {
		var trackData = {};

		if(typeof obj != 'undefined') {
			try {
				var story;

				// If it's an object then say no more... otherwise load the story
				if(typeof obj == 'object') {
					story = obj;
				} else if(typeof optionalData != 'undefined' && typeof optionalData.storyStore != 'undefined') {
					story = Ext.data.StoreManager.get(optionalData.storyStore).getById(obj); 	
				}
				else if(Pheme.app.currentStoryStore) {
			    	story = Ext.data.StoreManager.get(Pheme.app.currentStoryStore).getById(obj); 
				}

				// try to get story from central
				if(story == null || typeof story == 'undefined') {
					story = Ext.getStore('CentralStories').getById(obj);
				}				

				if(story == null || typeof story == 'undefined') {
					throw 'Story could not be loaded';
				} else {
					// build tracking data object
		            var mappedCategory = (story.data.ad_codes != undefined) ? story.data.ad_codes.ad : '';
		            var mappedCategories = mappedCategory.split("/");

		            trackData.distributor = Pheme.Config.fetchCurrentMarket().distributor || '';
		            trackData.newspaperBrand = Pheme.Config.fetchCurrentMarket().newspaperBrand || '';
		            trackData.pageName = mappedCategory;
		            trackData.storyTitle = story.data.title;
		            trackData.storyId = story.data.id;
		            
		            if(mappedCategories.length > 0) {
		                trackData.contentType = mappedCategories[mappedCategories.length-1];
		                trackData.section = mappedCategories[0];
		                if(mappedCategories.length > 2)
		                    trackData.subsection = mappedCategory.substring(0, mappedCategory.lastIndexOf("/"));
		            }
		        }
	        } catch(e) {
	        	console.log(e);
	        }
	    }

	    return trackData;
	},

	/*
	 *	Track an event on a page with no story related data (filter click, back, search...)
	 *	@param string name Name of event
	 *	@param string pageName page event took place
	 *	@param object optionalData Optional data params
	 */
	pageEvent: function(name, pageName, optionalData) {
		var trackData = {};
        trackData.distributor = Pheme.Config.fetchCurrentMarket().distributor;
        trackData.newspaperBrand = Pheme.Config.fetchCurrentMarket().newspaperBrand;
        trackData.pageName = pageName;

        // set optional data
        if(typeof optionalData != 'undefined') {
        	if(typeof optionalData.section != 'undefined')
        		trackData.section = optionalData.section;

        	if(typeof optionalData.searchTerm != 'undefined')
	            trackData.searchTerm = optionalData.searchTerm;
        }

        this.trackAction(name, trackData);
	},

	/*
	 *	Track a page view
	 *	@param string name Name of event
	 *	@param object obj Story Id or Story Object
	 */
	pageView: function(name, obj) {
		// get track data for story
		var trackData = this.getStoryTrackData(obj);
		
		// add config values
		trackData.distributor = Pheme.Config.fetchCurrentMarket().distributor;
	    trackData.newspaperBrand = Pheme.Config.fetchCurrentMarket().newspaperBrand;

	    // if story data has not set pageName set it
	    if(typeof trackData.pageName == 'undefined') {
	        trackData.pageName = name;
	    }

		this.trackState(name, trackData);
	},

	/*
	 *	Wrap the ADB.trackAction call in try/catch
	 */
	trackAction: function(name, data) {
		if(typeof ADB != 'undefined') {
			try {
				ADB.trackAction(name, data);
			} catch(e) {}
		}
	},

	/*
	 *	Wrap the ADB.trackState call in try/catch
	 */
	trackState: function(name, data) {
		if(typeof ADB != 'undefined') {
			try {
				ADB.trackState(name, data);
			} catch(e) {}
		}
	},

	/**
	 * [constructor description]
	 * @param  {[type]} config [description]
	 * @return {[type]}        [description]
	 */
    constructor: function(config) {
        this.initConfig(config);
    },
    /**
     * [init description]
     * @return {[type]} [description]
     */
    init: function() {
        this.callParent(arguments);
    }
});