/**
 * PolarAdsHelper
 * @class Pheme.helpers.PolarAdsHelper
 * Description: Helper class to manage and create Polar Native Ads
 */

Ext.define('Pheme.helpers.PolarAdsHelper', {
	singleton: true,
	config: {
        propertyKey: undefined
	},

	/**
	 * Downloads the SDK and inserts into DOM
	 */
	initSdk: function(propertyKey) {
		(function(d, s, id) {
			var js, fjs = d.getElementsByTagName(s)[0];
			if(d.getElementById(id)) { return; }
			js = d.createElement(s);
			js.id = id; js.type = "text/javascript"; js.async = true;
			js.src = "http://plugin.mediavoice.com/v0.5.83/plugin.js";
			fjs.parentNode.insertBefore(js, fjs);
		})(document, "script", "nativeads-plugin");

        if(Pheme.helpers.DfpAds.getSampleAds()) {
            this.setupSdk();
        } else {
            this.setupSdk(propertyKey); 
        }
	},

    /**
     * Setup SDK values
     */
    setupSdk: function(propertyKey) {
        propertyKey = propertyKey || "NA-TEST-11235814";
        this.setPropertyKey(propertyKey);
        window.NATIVEADS_QUEUE = window.NATIVEADS_QUEUE || [];
        window.NATIVEADS_QUEUE.push(['setPropertyID', this.getPropertyKey()]);
        window.NATIVEADS_QUEUE.push(['enableSPAMode', {pageOnHashChange: true}]);
        window.NATIVEADS_QUEUE.push(["logging.enable"]);
    },

    /**
     * Requests an Ad from the Polar Service
     * @param {[object]} unit [object containing data to form ad request]
     * @return {[boolean]} [returns false if param is undefined, true if request was completed]
     */

    requestAd: function(unit, label, store, model) {
        if(!unit) {
            return;
        }

        if(!store) {
            store = Ext.getStore('Stories');
        }

        if(!model) {
            model = Ext.create('Pheme.model.Story');
        }

        //This is for the messaging queue for the Polar SDK which is stored as a property in this class. Because of
        //the asynchronus nature of the Polar SDK, methods are stored in a queue to be loaded when the SDK has been
        //loaded into our application.
        var queue = window.NATIVEADS_QUEUE;

        var me = this;

        var onFillMethod = function(response) {
            me.onFill(response, Pheme.Config.getNativeAdSlot(), store, model);
        };

        if(Pheme.helpers.DfpAds.getDebugMode()) {
            alert('NATIVE AD REQUEST : ' + JSON.stringify(unit));
        }

        //This is the actual request to retrieve an ad. It's actually pushed into a queue based on Polar's
        //SDK for Single Page Applications. This will get fired when the SDK is in fact loaded into our application.
        queue.push(["requestAd", {
            "unit": unit,
            "label": label || "Pheme Native Ad",
            "onFill": onFillMethod,
            "onError": this.onError
        }]);

        return true;
    },

    /**
     * Fills an model with the response from an ad request. This method should not be called directly.
     * @param {[object]} response [This represents the response object returned from the Ad Service]
     * @param {[object]} index [The slot to insert the ad into]
     * @param {[object]} store [The store to put the ad into]
     * @param {[object]} model [object representing the Sencha Touch model to populate data from ad request]
     */
    onFill: function(response, index, store, model) {
        this.resetAdServer(window.NATIVEADS_QUEUE);

        var content = [];
        content.push(response.model.content);

        model.set('id', response.model.id);
        model.set('index', index);
        model.set('title', response.model.title);
        model.set('content', content);

        var prefix = '';
        if(typeof response.model.custom != 'undefined' && typeof response.model.custom.Type != 'undefined') {
            var prefixEndIndex = (response.model.custom.Type.indexOf('&') >= 0) ? response.model.custom.Type.indexOf('&') : response.model.custom.Type.length;
            prefix = response.model.custom.Type.substring(0, prefixEndIndex);
        }
        model.set('sponsor_prefix', prefix);

        model.set('sponsor_name', response.model.sponsor.name);

        var link = response.model.link;
        if(typeof response.model.custom != 'undefined' && typeof response.model.custom.Target_url != 'undefined') {
            if(response.model.custom.Target_url == '2.0') {
                link = response.model.link + '/appview';
            }
        }
        model.set('permalink', link);
        
        if(response.model.image != undefined && response.model.image.href != undefined) {
            model.set('featured_image_url_x2', response.model.image.href);
            model.set('featured_image_url', response.model.image.href);
            model.set('thumbnail_url_x2', response.model.image.href);
            model.set('thumbnail_url', response.model.image.href);
        }
        model.internalId = response.model.id;
        model.isNativeAd = true;
        model.adModel = response.model;
        model.adTag = response.adtag;

        var dateRange = this.getDateRangeFromIndex(index, store);
        model = this.setDateBasedOnIndexInsert(index, model, dateRange);

        // clear existing native ads from store
        store.each(function(item, index, length) {
            if(typeof item.isNativeAd != 'undefined' && item.isNativeAd) {
                store.remove(item);
            }
        });

        store.add(model);

        Pheme.helpers.Common.setStoryStoreRecordIndexes(store);
        this.refreshListView();

        //Called twice to ensure ad will get filled.
        this.resetAdServer(window.NATIVEADS_QUEUE);
    },

    /**
     * Destroys ad model in the event that the Ad Request was unsuccessful.
     * @param {[object]} error [This represents the error object returned from the Ad Service]
     */
    onError: function(error) {
        return;
    },

    /**
     * Gets a range of dates in between two stories based on the index parameter
     * @param {[int]} index
     * @param {[object]} store [The store to search in. This is optional. The default is to use the 'Stories' store]
     * @returns {Array} [Contains the date of the story previous to the index and the date of the story at the index]
     */
    getDateRangeFromIndex: function(index, store) {
        if(!store) {
            store = Ext.getStore('Stories');
        }

        var dateRange = [];

        // catch a list that does not have more that 'index' stories
        if(store.getCount() < index - 1) {
            index = store.getCount() - 1;
        }

        dateRange.push(store.getAt(index).data.date_sort);
        dateRange.push(store.getAt(index - 1).data.date_sort);

        return dateRange;
    },

    /**
     * Sets the date of a Story model to one second after the date of the placement index.
     * @param {[int]} index
     * @param {[object]} model
     * @param {[array]} dateRange
     * @returns {[object]} model
     */
    setDateBasedOnIndexInsert: function(index, model, dateRange) {
        if(!index || !model || !dateRange) {
            return;
        }

        var earlyDate = dateRange[0].getTime(),
            formattedDate = new Date(earlyDate + 1000);

        model.set('date_sort', formattedDate);

        return model;
    },

    /**
     * Track primary click event
     * @param  {[type]} model [description]
     * @param  {[type]} adtag [description]
     * @param  {[type]} unit  [description]
     * @param  {[type]} label [description]
     * @return {[type]}       [description]
     */
    trackTapEvent: function(model, adtag, unit, label) {
        if (!model || !adtag) {
            return;
        }

        window.NATIVEADS_QUEUE.push(["event.track", {
            "kind"  : "primary-click",
            "unit"  : unit,
            "model" : model,
            "adtag" : adtag,
            "label" : label
        }]);
    },

    refreshListView: function() {
        var mainList = Ext.getCmp('storiesList');
        mainList.setScrollToTopOnRefresh(false);
        mainList.refresh();
    },

    resetAdServer: function(queue) {
        queue.push(['resetAdServer']);
    }
});
